#include "stdafx.h"

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <math.h>
#include <time.h>
#include <time.h>

using namespace std;

#define MAIN
#include "ldraw.hpp"

void pause(void)
{
	cout << "Press <Enter> to quit" << endl;
	getchar();
}
void usage(void)
{
	cout << "Unificator [options] InFile, Outfile" << endl;
	cout << "-ut <val>: Vertices unification threshold (default 0.005)" << endl;
	cout << "-s: Snap on subfiles" << endl;
	cout << "-us <val>: Unification on subfiles threshold (default 0.005)" << endl;
	cout << "-l \"path\": define ldraw search path" << endl;
	pause();
}

int main(int argc, char *argv[])
{
	printf("\nLdraw Unificator 1.0 - by Philo");
	printf("\n-------------------------------\n");

	if(argc<3) 
	{
		usage();
		return 1;
	}
	bool SubSnap=false;

	LdPath* path=new LdPath();
	// Get input file path
	string curpath=argv[argc-2];
	size_t pathlen = curpath.find_last_of("\\");
	if(pathlen == string::npos) curpath = ".\\"; else curpath.resize(pathlen+1);
	path->addPath(curpath);
	string derivedpath=curpath;
	path->addPath(derivedpath.append("p\\"));
	derivedpath=curpath;
	path->addPath(derivedpath.append("s\\"));

	for(int i=1; i<argc-2; i++)
	{
		if(argv[i][0] != '-')
		{
			usage();
			return 1;
		}
		switch (argv[i][1])
		{
		case 'u':
			switch (argv[i][2])
			{
			case 't':
				i++;
				sscanf(argv[i], "%lf",&UnifyDist);
				break;
			case 's':
				i++;
				sscanf(argv[i], "%lf",&SubSnapDist);
				break;
			default:
				usage();
				return 1;
			}
			break;
		case 's':
			SubSnap = true;
			break;

		case 'l':
			i++;
			curpath = argv[i];
			if(curpath.at(curpath.size()-1) != '\\') 
			{
				curpath.append("\\");
			}
			derivedpath=curpath;
			path->addPath(derivedpath.append("p\\"));
			derivedpath=curpath;
			path->addPath(derivedpath.append("parts\\"));
			path->addPath(derivedpath.append("s\\"));
			curpath.append("Unofficial\\");
			derivedpath=curpath;
			path->addPath(derivedpath.append("p\\"));
			derivedpath=curpath;
			path->addPath(derivedpath.append("parts\\"));
			path->addPath(derivedpath.append("s\\"));

			break;
		default:
			usage();
			return 1;
		}
	}

	cout << "Vertices unification threshold: " << UnifyDist << endl;
	if(SubSnap)	cout << "Unification on subfiles threshold: " << SubSnapDist << endl;
	
	// Inlined = true;

	Fldraw infile;
	cout << endl << "Reading input file " << argv[argc-2] << endl;
	if(!infile.ldLoad (argv[argc-2]))
	{
		cout << argv[argc-2] << " not found" << endl;
		pause();
		return 2;
	}


	if(SubSnap)
	{
		cout << endl << "LDraw subfiles search locations:\n" << path->toString().c_str() << endl;
		cout << "\nExpanding input file subfiles\n";
		infile.ldExpand(path);
		cout << endl;
	}
	cout << endl << "Unifying input file vertices" << endl;
	infile.Unify();
	
	cout << endl << "Writing output file " << argv[argc-1] << endl;
	if(!infile.ldSave(argv[argc-1]))
	{
		cout <<" Can't create" << argv[argc-1] << endl;
		pause();
		return 2;
	}
	pause();
	return 0;
}

