#include "stdafx.h"
#include "ldraw.hpp"


// Class Point
	Point::Point()
	{
		//printf("constructeur point\n");
		m_x=0;
		m_y=0;
		m_z=0;
	}

	Point::Point(const Point &p)
	{
		m_x=p.m_x;
		m_y=p.m_y;
		m_z=p.m_z;
	}

	Point::Point(double x, double y, double z )
	{
		m_x=x;
		m_y=y;
		m_z=z;
	}

	
	Point::addp(Point* p)
	{
		p->m_x += m_x;
		p->m_y += m_y;
		p->m_z += m_z;
	}

	void Point::setp (double xi, double yi, double zi)
	{
		m_x=xi;
		m_y=yi;
		m_z=zi;
	}
	void Point::setp (const Point &p)
	{
		m_x=p.m_x;
		m_y=p.m_y;
		m_z=p.m_z;
	}

	void Point::swapXY()
	{
		double temp;
		temp = m_x;
		m_x = m_y;
		m_y = temp;
	}

	void Point::swapYZ()
	{
		double temp;
		temp = m_z;
		m_z = m_y;
		m_y = temp;
	}

	char* Point::pparse (char* str)
	{
		char *p;
		p=str;
		if(sscanf(p,"%lf", &m_x) == 0) return NULL;
		// eat front spaces
		while(isspace(*p)!=0) p++;
		// find next space
		while(isspace(*p)==0) p++;
		if(sscanf(p,"%lf", &m_y) == 0) return NULL;
		// eat front spaces
		while(isspace(*p)!=0) p++;
		// find next space
		while(isspace(*p)==0) p++;
		if(sscanf(p,"%lf", &m_z) == 0) return NULL;
		// eat front spaces
		while(isspace(*p)!=0) p++;
		// find next space
		while(isspace(*p)==0) p++;
		return p;
	}

	double Point::distance (Point &p)
	{
		// Euclidian distance
		// return sqrt((p.m_x-m_x)*(p.m_x-m_x) + (p.m_y-m_y)*(p.m_y-m_y) + (p.m_z-m_z)*(p.m_z-m_z));
		// Manhattan distance
		return fabs(p.m_x-m_x) + fabs(p.m_y-m_y) + fabs(p.m_z-m_z);
	}

	// returns true if two points are close enough
	bool Point::compare(Point &p)
	{
		CompPoint++;
		if(fabs(p.m_x-m_x)>LdPrecision) return false;
		if(fabs(p.m_y-m_y)>LdPrecision) return false;
		if(fabs(p.m_z-m_z)>LdPrecision) return false;
		return true;
	}
	
	string Point::toString()
	{
		char buf[100];
		sprintf(buf,"%.7lg %.7lg %.7lg", m_x, m_y, m_z);
		return buf;
	}

	double Point::getYOrientation(Point &p1, Point &p3)
	{
		return ((p3.m_x - m_x)*(p1.m_z - m_z) - (p1.m_x - m_x)*(p3.m_z - m_z));
	}

	double Point::getLineDist(Point &p1, Point &p2)
	{
		double d;
		d= -((p2.m_x - m_x)*(p1.m_z - m_z) - (p1.m_x - m_x)*(p2.m_z - m_z))/sqrt((p2.m_x - p1.m_x)*(p2.m_x - p1.m_x) + (p2.m_z - p1.m_z)*(p2.m_z - p1.m_z));
		return d;
	}


	void Point::printclassname()
	{
		cout << "classe point" << endl;
	}

	

//  == overload to compare "close enough" points
bool operator == (Point &p1, Point &p2)
{
	return (p1.compare(p2));
//	return (p1.distance(p2)<LdPrecision);
}

// Class Vector

Vector::Vector(Point &p1, Point &p2)
{
	m_x = p2.getx() - p1.getx();
	m_y = p2.gety() - p1.gety();
	m_z = p2.getz() - p1.getz();
}

Vector* Vector::cross (Vector &v)
{
	double x,y,z;
	x = m_y*v.m_z - m_z*v.m_y;
	y = m_z*v.m_x - m_x*v.m_z;
	z = m_x*v.m_y - m_y*v.m_x;
	return new Vector(x,y,z);
}

double Vector::dot (Vector &v)
{
	return ( m_x*v.m_x + m_y*v.m_y + m_z*v.m_z );
}

double Vector::norm ()
{
	return sqrt( m_x*m_x + m_y*m_y + m_z*m_z );
}

void Vector::mult(double k)
{
	m_x=m_x*k;
	m_y=m_y*k;
	m_z=m_z*k;
}

void Vector::normalize ()
{
	double length;
	length=norm();
	if (length < EPSILON) return;
	m_x /= length;
	m_y /= length;
	m_z /= length;
}

double Vector::perp(Vector &v)
{
	return m_x*v.m_z - m_z*v.m_x;
}

// Class Matrix

Matrix::Matrix()
{
	m_mat[0][0]=0;
	m_mat[0][1]=0;
	m_mat[0][2]=0;
	m_mat[1][0]=0;
	m_mat[1][1]=0;
	m_mat[1][2]=0;
	m_mat[2][0]=0;
	m_mat[2][1]=0;
	m_mat[2][2]=0;
}

Matrix::Matrix(double a,double b,double c,double d,double e,double f,double g,double h,double i)
{
	m_mat[0][0]=a;
	m_mat[0][1]=b;
	m_mat[0][2]=c;
	m_mat[1][0]=d;
	m_mat[1][1]=e;
	m_mat[1][2]=f;
	m_mat[2][0]=g;
	m_mat[2][1]=h;
	m_mat[2][2]=i;
}

Matrix::Matrix(Point &p1, Point &p2, Point &p3)
{
	m_mat[0][0]=p1.getx();
	m_mat[0][1]=p1.gety();
	m_mat[0][2]=p1.getz();
	m_mat[1][0]=p2.getx();
	m_mat[1][1]=p2.gety();
	m_mat[1][2]=p2.getz();
	m_mat[2][0]=p3.getx();
	m_mat[2][1]=p3.gety();
	m_mat[2][2]=p3.getz();
}

string Matrix::toString()
{
	char buf[100];
	sprintf(buf,"%.7lg %.7lg %.7lg %.7lg %.7lg %.7lg %.7lg %.7lg %.7lg", m_mat[0][0], m_mat[0][1], m_mat[0][2], m_mat[1][0], m_mat[1][1], m_mat[1][2], m_mat[2][0], m_mat[2][1], m_mat[2][2]);
	return buf;
}

Vector* Matrix::Mult(Vector &v1)
{
	double x,y,z;
	x =  m_mat[0][0]*v1.getx() +  m_mat[0][1]*v1.gety() + m_mat[0][2]*v1.getz();
	y =  m_mat[1][0]*v1.getx() +  m_mat[1][1]*v1.gety() + m_mat[1][2]*v1.getz();
	z =  m_mat[2][0]*v1.getx() +  m_mat[2][1]*v1.gety() + m_mat[2][2]*v1.getz();
	return new Vector(x,y,z);
}

Matrix* Matrix::MatMult(Matrix &m1)
{
	Matrix* m=new Matrix(0,0,0,0,0,0,0,0,0);
	for(int i=0; i<3; i++)
	{
		for(int j=0; j<3; j++)
		{
			for(int k=0; k<3; k++)
			{
				m->m_mat[i][j] += m_mat[i][k] * m1.m_mat[k][j];
			}
		}
	}
	return m;
}

double Matrix::MatDeterminant()
{
	return m_mat[0][0]*m_mat[1][1]*m_mat[2][2] + m_mat[1][0]*m_mat[2][1]*m_mat[0][2] + m_mat[2][0]*m_mat[0][1]*m_mat[1][2]
		- m_mat[2][0]*m_mat[1][1]*m_mat[0][2] - m_mat[0][0]*m_mat[2][1]*m_mat[1][2] - m_mat[1][0]*m_mat[0][1]*m_mat[2][2];
}

void Matrix::flattenY(void)
{
	m_mat[1][0] = 0;
	m_mat[1][1] = 1;
	m_mat[1][2] = 0;
}
void Matrix::projectY(Vector *normal)
{
	// Y matrix coefficients
	m_mat[1][0]= -(m_mat[0][0] * normal->getx() + m_mat[2][0] * normal->getz())/normal->gety();
	m_mat[1][1]= 1;
	m_mat[1][2]= -(m_mat[0][2] * normal->getx() + m_mat[2][2] * normal->getz())/normal->gety();
		
}

void Matrix::swapXY()
{
	double temp;
	temp = m_mat[1][0]; m_mat[1][0] = m_mat[0][0]; m_mat[0][0] = temp;
	temp = m_mat[1][1]; m_mat[1][1] = m_mat[0][1]; m_mat[0][1] = temp;
	temp = m_mat[1][2]; m_mat[1][2] = m_mat[0][2]; m_mat[0][2] = temp;
}
void Matrix::swapYZ()
{
	double temp;
	temp = m_mat[1][0]; m_mat[1][0] = m_mat[2][0]; m_mat[2][0] = temp;
	temp = m_mat[1][1]; m_mat[1][1] = m_mat[2][1]; m_mat[2][1] = temp;
	temp = m_mat[1][2]; m_mat[1][2] = m_mat[2][2]; m_mat[2][2] = temp;
}

// Class LdObject

bool LdObject::Overlap(LdObject* surf)
{
	LdLine *ldl = (LdLine *)(surf->getFirstLine());
	do
	{
		if(isOutside(ldl)) 
		{
			delete ldl;
			return false;
		}
		delete ldl;
	} while ((ldl = (LdLine *)(surf->getNextLine()))!=NULL);
	if(getType() == Subfile) return true;
	if((ldl = (LdLine *)(getFirstLine()))==NULL) return false;
	if(getType() == Line)
	{
		if(surf->isOutside(ldl))
		{
			delete ldl;
			return false;
		}
		ldl->InvertWinding();
		if(surf->isOutside(ldl)) 
		{
			delete ldl;
			return false;
		}
		delete ldl;
	}
	else do
	{
		if(surf->isOutside(ldl)) 
		{
			delete ldl;
			return false;
		}
		delete ldl;
	} while ((ldl = (LdLine *)(getNextLine()))!=NULL);
	return true;
}

// class PointBary

PointBary::PointBary(Point &p, bool fixed)
{
	m_xsum = p.getx();
	m_ysum = p.gety();
	m_zsum = p.getz();
	nbpoint=1;
	m_x=m_xsum;
	m_y=m_ysum;
	m_z=m_zsum;
	m_fixed=fixed;
}

void PointBary::addPoint(Point &p, bool fixed)
{
	if(m_fixed) return;
	if(fixed)
	{
		m_xsum = p.getx();
		m_ysum = p.gety();
		m_zsum = p.getz();
		nbpoint=1;
		m_fixed=true;
	}
	else
	{
		m_xsum += p.getx();
		m_ysum += p.gety();
		m_zsum += p.getz();
		nbpoint++;
	}
	m_x=m_xsum/nbpoint;
	m_y=m_ysum/nbpoint;
	m_z=m_zsum/nbpoint;
}

// class PointUnify
PointUnify::~PointUnify()
{
	for (int i=0; i<barytab.size(); i++)
	{
		if(barytab[i] != NULL) delete barytab[i];
	}

}

void PointUnify::addPoint(Point &p, bool fixed)
{
	for (int i=0; i<barytab.size(); i++)
	{
		double dist;
		if(barytab[i]->isFixed()) dist = LdPrecision; else dist = UnifyDist;
		if(barytab[i]->distance(p) < dist)
		{
			barytab[i]->addPoint(p,fixed);
			return;
		}
	}
	barytab.push_back(new PointBary(p, fixed));
}

void PointUnify::getPoint(Point &p)
{
	double curdist, mindist=100000;
	int index;
	if(barytab.size()==0) return;
	
	for (int i=0; i<barytab.size(); i++)
	{
		if((curdist = barytab[i]->distance(p)) < mindist)
		{
			mindist=curdist;
			index = i;
		}
	}
//	printf("\n%d, %lf, %d", index, mindist, barytab.size());
	if(barytab[index]->isFixed())
	{
		if(barytab[index]->distance(p) < SubSnapDist)
		{
			p.setp(*(Point*)barytab[index]);
			return;
		}

	}
	else
	{
		if(barytab[index]->distance(p) < UnifyDist)
		{
			p.setp(*(Point*)barytab[index]);
			return;
		}
	}
}

string PointUnify::toString()
{
	string result;
	for (int i=0; i<barytab.size(); i++)
	{
		result += barytab[i]->toString();
		result +="\n";
	}
	return result;
}


// Class Poly
void Poly::addNewVertex(Point *newp)
{
	for(int i=0; i<Vertices.size(); i++)
	{
		// Check if vertex already exists
		if(*newp == *(Vertices[i]->p)) return;
	}
	Point *q;
	q=new Point(*newp);
	Polyvertex* vertex = new Polyvertex;
	vertex->p=q;
	vertex->azimuth=0;
	Vertices.push_back(vertex);
}

void Poly::getBottomMost(void)
{
	int bottommost=0;
	double zmin=100000000;
	for(int i=0; i<Vertices.size(); i++)
	{
		double curz;
		if((curz = Vertices[i]->p->getz()) <= zmin)
		{
			if((curz<zmin) || ((curz=zmin) && (Vertices[i]->p->getx()<Vertices[bottommost]->p->getx())))
			{
				zmin=curz;
				bottommost=i;
			}
		}
	}
	Polyvertex *v;
	v=Vertices[0];
	Vertices[0]=Vertices[bottommost];
	Vertices[bottommost]=v;
}

void Poly::setAzimuth(void)
{
	double x0=Vertices[0]->p->getx();
	double z0=Vertices[0]->p->getz();
	for(int i=1; i<Vertices.size(); i++)
	{
		double x,z;
		double az;
		x=Vertices[i]->p->getx()-x0;
		z=Vertices[i]->p->getz()-z0;
		if(z<0) z=0;
		az=atan2(x,z);
		Vertices[i]->azimuth = az;
	}
}


void Poly::sortAz(void)
{
	Polyvertex *v;
	Point P0(*(Vertices[0]->p));
	for(int i=1; i<Vertices.size()-1; i++)
	{
		double a,b;
		for(int j=i+1; j<Vertices.size(); j++)
		{
			// cout << i << " " << j << " " <<Vertices[i]->azimuth << " " << Vertices[j]->azimuth;
			if(fabs((a=Vertices[i]->azimuth) - (b=Vertices[j]->azimuth))<LdPrecision)
			{
				if(a==-1000) continue;
				if(P0.distance(*(Vertices[i]->p)) < P0.distance(*(Vertices[j]->p)))
				{
					v=Vertices[i];
					Vertices[i]=Vertices[j];
					Vertices[j]=v;
					// cout << " sameaz"; 
				}
				Vertices[j]->azimuth=-1000;
			} 
			else 
			if((a=Vertices[i]->azimuth) < (b=Vertices[j]->azimuth))
			{
				v=Vertices[i];
				Vertices[i]=Vertices[j];
				Vertices[j]=v;
				// cout <<" azswap";
			}
			// cout << endl << toString().c_str()<<endl;
		}
	}
	for(i=Vertices.size()-1; i>0; i--)
	{
		if(Vertices[i]->azimuth==-1000)
		{
			delete Vertices[i];
			Vertices.pop_back();
		}
	}
}

void Poly::getConvexHull(void)
{
	Polyvertex *v;
	v=Vertices[0];
	Vertices.push_back(v);
	for(int i=0; i<Vertices.size()-2; i++)
	{
		// cout << "cvx " << i << endl;
		// cout << toString().c_str();
		if(Vertices[i]==NULL) continue;
		if(Vertices[i]->p->getYOrientation(*(Vertices[i+1]->p), *(Vertices[i+2]->p)) > -LdPrecision)
		{
			delete Vertices[i+1]->p;
			delete Vertices[i+1];
			for(int j=i+2; j<Vertices.size(); j++)
			{
				Vertices[j-1]=Vertices[j];
			}
			Vertices.pop_back();
			i--;
		}
		// cout << i << " " << a << endl << endl;
	}
	Vertices.pop_back();
	if(Vertices[Vertices.size()-1]->p->getYOrientation(*(Vertices[0]->p), *(Vertices[1]->p)) > -LdPrecision)
	{
		delete Vertices[0]->p;
		delete Vertices[0];
		for(int j=1; j<Vertices.size(); j++)
		{
			Vertices[j-1]=Vertices[j];
		}
		Vertices.pop_back();
	}

	return;
}

LdObject* Poly::getLdSurf(int color)
{
	
	LdObject* ldo;
	int n=Vertices.size();
	if(n<3)
	{
		for(int i=0; i<n; i++)
		{
			delete Vertices[i]->p;
			delete Vertices[i];
		}
		Vertices.clear();
		return NULL;
	}
	if(n==3)
	{
		ldo=(LdObject*)(new LdTriangle(*Vertices[0]->p, *Vertices[1]->p, *Vertices[2]->p, color));
		delete Vertices[2]->p;
		delete Vertices[2];
		Vertices.pop_back();
		return ldo;
	}
	n--;
	ldo=(LdObject*)(new LdQuad(*Vertices[n-2]->p, *Vertices[n-1]->p, *Vertices[n]->p, *Vertices[0]->p, color));
	delete Vertices[n]->p;
	delete Vertices[n];
	delete Vertices[n-1]->p;
	delete Vertices[n-1];
	Vertices.pop_back();
	Vertices.pop_back();
	return ldo;
}

string Poly::toString(void)
{
	string result;
	char buf[10];
	Point p;
	result="";
	for(int i=0; i<Vertices.size(); i++)
	{
		if(Vertices[i]==NULL) 
		{
			result += "NULL\n";
			continue;
		}
		p=*(Vertices[i]->p);
		result += Vertices[i]->p->toString();
		sprintf(buf, " %g", Vertices[i]->azimuth);
		result += buf;
		result += "\n";
	}
	return result;
}

// Class LDComment

LdComment::LdComment(const LdComment &ldc)
{
	m_cs=ldc.m_cs;
}

LdComment::LdComment(const string &str)
{
	m_cs=str;
}

LdComment::LdComment(const char *str)
{
	m_cs=str;
}

string LdComment::toString()
{
	string result = "0 ";
	result += m_cs;
	return result;
}

int LdComment::readbfc()
{
	char comment[1000];
	strcpy(comment, m_cs.c_str());
	for(int i=0; i<strlen(comment); i++) comment[i]=toupper(comment[i]);
	if(strstr(comment,"BFC") == NULL) return 0;
	if(strstr(comment,"INVERTNEXT") != NULL) return 1;
	if(strstr(comment,"CERTIFY") == NULL) return 0;
	if(strstr(comment,"CCW") != NULL) return 2;
	if(strstr(comment,"CW") != NULL) return 3;
	return 2;
}

// Class LDSubfile

LdSubfile::LdSubfile(const char *str)
{
	char*p;
	Point p1,p2,p3;
	p=(char*)str;
	// eat front spaces
	while(isspace(*p)!=0) p++;
	// find next space after type
	while(isspace(*p)==0) p++;
	sscanf(p,"%d", &m_col);
	// skip color
	while(isspace(*p)!=0) p++;
	// find next space
	while(isspace(*p)==0) p++;
	// Read 1st point
	p=m_origin.pparse(p);
	p=p1.pparse(p);
	p=p2.pparse(p);
	p=p3.pparse(p);
	m_mat=Matrix(p1,p2,p3);
	while(isspace(*p)!=0) p++;
	m_file=p;
}

bool LdSubfile::isOutside(LdObject* line)
{
	Point* p1;
	Point* p2;
	p1=((LdLine*)line)->getP1();
	p2=((LdLine*)line)->getP2();
	if(m_origin.getYOrientation(*p1,*p2) > 0) return true;
	return false;
}

LdSubfile::LdSubfile(const LdSubfile &lds)
{
	m_col=lds.m_col;
	m_origin=lds.m_origin;
	m_mat=lds.m_mat;
	m_file=lds.m_file;
}


LdSubfile::translate(Point &p)
{
	p.addp(&m_origin);
}

LdSubfile::transform(Matrix &m)
{
	Vector* v1;
	v1=m.Mult((Vector)m_origin);
	m_origin=*v1;
	delete v1;
	Matrix* m2;
	m2=m.MatMult(m_mat);
	m_mat=*m2;
	delete m2;
}

void LdSubfile::flattenY(void)
{
	m_origin.flattenY();
	m_mat.flattenY();
}

void LdSubfile::swapXY()
{
	m_origin.swapXY();
	m_mat.swapXY();
}
void LdSubfile::swapYZ()
{
	m_origin.swapYZ();
	m_mat.swapYZ();
}

void LdSubfile::projectY(LdObject *surf) 
{
	Vector *v;
	if(!(surf->isSurface())) return;
	surf->projectPointY(m_origin);
	v=surf->getNormal();
	m_mat.projectY(v);
	delete v;
}

string LdSubfile::toString()
{
	char color[20];
	string result = "1 ";
	sprintf(color,"%d ",m_col);
	result += color;
	result += m_origin.toString();
	result += " ";
	result += m_mat.toString();
	result += " ";
	result += m_file;
	return result;
}


// Class LdLine

LdLine::LdLine(const LdLine &l)
{
	m_p1=l.m_p1;
	m_p2=l.m_p2;
	m_col=l.m_col;
}

LdLine::LdLine(const char *str)
{
	char*p;
	p=(char*)str;
	// eat front spaces
	while(isspace(*p)!=0) p++;
	// find next space after type
	while(isspace(*p)==0) p++;
	sscanf(p,"%d", &m_col);
	// skip color
	while(isspace(*p)!=0) p++;
	// find next space
	while(isspace(*p)==0) p++;
	// Read 1st point
	p=m_p1.pparse(p);
	p=m_p2.pparse(p);
}

LdLine::LdLine(const Point &p1, const Point &p2, const int col)
{
	m_p1=p1;
	m_p2=p2;
	m_col=col;
}

string LdLine::toString()
{
	char color[20];
	string result = "2 ";
	sprintf(color,"%d ",m_col);
	result += color;
	result += m_p1.toString();
	result += " ";
	result += m_p2.toString();
	return result;
}

LdLine::translate(Point &p)
{
	p.addp(&m_p1);
	p.addp(&m_p2);
}

void LdLine::addPoints (PointUnify &PU, bool fixed)
{
	PU.addPoint(m_p1, fixed);
	PU.addPoint(m_p2, fixed);
}
void LdLine::getPoints (PointUnify &PU)
{
	PU.getPoint(m_p1);
	PU.getPoint(m_p2);
}


LdLine::transform(Matrix &m)
{
	Vector* v1;
	v1=m.Mult((Vector)m_p1);
	m_p1=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p2);
	m_p2=*v1;
	delete v1;
}

void LdLine::swapXY()
{
	m_p1.swapXY();
	m_p2.swapXY();
}
void LdLine::swapYZ()
{
	m_p1.swapYZ();
	m_p2.swapYZ();
}

void LdLine::InvertWinding()
{
	Point p;
	p = m_p1;
	m_p1 = m_p2;
	m_p2 = p;
}

bool LdLine::isInvalid(LdObject* &ldo)
{
	if(m_p1.distance(m_p2) < EPSILON)
	{
		ldo=new LdEmpty;
		return true;
	}
	return false;
}


void LdLine::flattenY()
{
	m_p1.flattenY();
	m_p2.flattenY();
}

void LdLine::projectY(LdObject *surf)
{
	if(!(surf->isSurface())) return;
	surf->projectPointY(m_p1);
	surf->projectPointY(m_p2);
}

LdObject* LdLine::getFirstLine()
{
	return new LdLine(m_p1, m_p2);
}

bool LdLine::isOutside(LdObject* line)
{
	Point* p1;
	Point* p2;
	p1=((LdLine*)line)->getP1();
	p2=((LdLine*)line)->getP2();
	if((m_p1.getYOrientation(*p1,*p2) > 0) && 
		(m_p2.getYOrientation(*p1,*p2) > 0)) return true;
	return false;
}

// Code adapted from softSurfer (www.softsurfer.com)  Copyright 2001
bool LdLine::inter (LdLine *line, Point *p)
{
	Vector u(m_p1, m_p2);
	Vector v(line->m_p1, line->m_p2);
	Vector w(m_p1, line->m_p1);
	double d=u.perp(v);
	if(fabs(d)<LdPrecision) return false; // parallel lines
    double sI = w.perp(v)/d;
    if (sI < 0 || sI > 1) return false; // no intersect in segment
	Point result(m_p1);
	u.mult(sI);
	u.addp(&result);
    p->setp(result);                   // compute intersect point
	return true;
}

bool LdLine::inter (Point* p1, Point* p2, Point *p)
{
	Vector u(m_p1, m_p2);
	Vector v(*p1, *p2);
	Vector w(m_p1, *p1);
	double d=u.perp(v);
	if(fabs(d)<LdPrecision) return false; // parallel lines
    double sI = w.perp(v)/d;
    if (sI < 0 || sI >= 1) return false; // no intersect in segment
	Point result(m_p1);
	u.mult(sI);
	u.addp(&result);
    p->setp(result);                   // compute intersect point
	return true;
}


void LdLine::Cut(LdObject* line, LdObject* &o1, LdObject* &o2)
{

	int nbside = 2;

	Point p1, p2, p3;
	p1=*((LdLine*)line)->getP1();
	p2=*((LdLine*)line)->getP2();

	// calculate distances between surface vertices and cutting line
	dist[0] = m_p1.getLineDist(p1,p2);
	dist[1] = m_p2.getLineDist(p1,p2);

	// find number of points on the cutting line, one side, or on the other
	int distnul=0;
	int distpos=0;
	

	for(int i=0; i<nbside; i++)
	{
		if(fabs(dist[i])<LdPrecision) distnul++;
		if(dist[i]>LdPrecision) distpos++;
	}
	int distneg=nbside-distnul-distpos;

	// all points on the wrong side, surface deleted
	if(distpos==0) return;
	// all points on the right side, or on line: surface not modified
	if((distneg==0)) 
	{
		o1=new LdLine(*this);
		return;
	}
	Point p;

	if(inter((LdLine*)line, &p))
	{
		if(dist[1] > 0)
			o1=new LdLine(m_p2,p, m_col);	
		else o1=new LdLine(m_p1,p, m_col);
	}
	return;
}



bool LdLine::matchLine(LdObject &line)
{
/*	if(line.getType() != Line)
	{
		cout << "Trying to use matchline with a non-line" << endl;
		return false;
	}
*/	
	if((m_p1 == ((LdLine&)line).m_p1) && (m_p2 == ((LdLine&)line).m_p2)) return true;
	if((m_p2 == ((LdLine&)line).m_p1) && (m_p1 == ((LdLine&)line).m_p2)) return true;

/* // Following code is not more efficient
	if(m_p1 == ((LdLine&)line).m_p1)
	{
		if(m_p2 == ((LdLine&)line).m_p2) return true;
		return false;
	}
	else if(m_p2 == ((LdLine&)line).m_p1)
	{
		if(m_p1 == ((LdLine&)line).m_p2) return true;
		return false;
	}
*/
	return false;
}

bool LdLine::matchPoint(Point &p1)
{
	if((m_p1==p1) || (m_p2==p1)) return true;
	return false;
}

// Class LdTriangle

LdTriangle::LdTriangle(const LdTriangle &t)
{
	m_p1=t.m_p1;
	m_p2=t.m_p2;
	m_p3=t.m_p3;
	m_col=t.m_col;
}

LdTriangle::LdTriangle(Point &p1, Point &p2, Point &p3, int col)
{
	m_p1=p1;
	m_p2=p2;
	m_p3=p3;
	m_col=col;
}

LdTriangle::LdTriangle(const char *str)
{
	char*p;
	p=(char*)str;
	// eat front spaces
	while(isspace(*p)!=0) p++;
	// find next space after type
	while(isspace(*p)==0) p++;
	sscanf(p,"%d", &m_col);
	// skip color
	while(isspace(*p)!=0) p++;
	// find next space
	while(isspace(*p)==0) p++;
	// Read 1st point
	p=m_p1.pparse(p);
	p=m_p2.pparse(p);
	p=m_p3.pparse(p);
}

string LdTriangle::toString()
{
	char color[20];
	string result = "3 ";
	sprintf(color,"%d ",m_col);
	result += color;
	result += m_p1.toString();
	result += " ";
	result += m_p2.toString();
	result += " ";
	result += m_p3.toString();
	return result;
}


LdTriangle::translate(Point &p)
{
	p.addp(&m_p1);
	p.addp(&m_p2);
	p.addp(&m_p3);
}

void LdTriangle::swapXY()
{
	m_p1.swapXY();
	m_p2.swapXY();
	m_p3.swapXY();
}
void LdTriangle::swapYZ()
{
	m_p1.swapYZ();
	m_p2.swapYZ();
	m_p3.swapYZ();
}


void LdTriangle::RotPoints(void)
{
	Point temp;
	double disttemp;
	temp = m_p1;
	m_p1=m_p2;
	m_p2=m_p3;
	m_p3=temp;
	disttemp=dist[0];
	dist[0]=dist[1];
	dist[1]=dist[2];
	dist[2]=disttemp;
}

LdTriangle::transform(Matrix &m)
{
	Vector* v1;
	v1=m.Mult((Vector)m_p1);
	m_p1=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p2);
	m_p2=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p3);
	m_p3=*v1;
	delete v1;
}

LdObject* LdTriangle::getFirstLine()
{
	m_edgecpt=0;
	return new LdLine(m_p1,m_p2);
}

LdObject* LdTriangle::getNextLine()
{
	m_edgecpt++;
	switch (m_edgecpt)
	{
	case 1:
		return new LdLine(m_p2,m_p3);
	case 2:
		return new LdLine(m_p3,m_p1);
	default:
		return NULL;
	}
}

/*
bool LdTriangle::projectPointY(Point &p)
{
	Point q(p.getx(), p.gety()+1, p.getz()); 
	compute plane equation of triangle(V0,V1,V2) 
	Vector u(m_p2, m_p1);
	Vector v(m_p3, m_p1);
	Vector* n;
	n=u.cross(v);
	double d1=-n->dot((Vector)m_p1);
  // plane equation 1: n.X+d1=0 

  // put S0, S1 into plane equation 1 to compute signed distances to the plane
	double du0= n->dot((Vector)p)+d1;
	double du1= n->dot((Vector)q)+d1;

	// coplanarity robustness check 
	if(fabs(du0)<EPSILON) du0=0.0;
	if(fabs(du1)<EPSILON) du1=0.0;
	double du0du1=du0*du1;

	if(n->distance() < EPSILON) {delete n; return false;} // triangle is degenerate

	//  *idx=(du0>0)?0:1; // Which segment end is "outside" triangle?
	Vector dir(q,p); // ray direction vector
	Vector w0(p, m_p1);
	double a=n->dot(w0);
	double b=n->dot(dir);
	if (fabs(b) < EPSILON) {delete n; return false;} // ray is parallel to triangle plane

	double r = a / b;
	dir.mult(r);
	Point result=p;
	((Point)dir).addp(&result);

	Vector w(m_p1);
	w.mult(-1.);
	result.addp((Point*)&w);

    p=result; // I is in T
	delete n;
	return true;
}

*/
bool LdTriangle::projectPointY(Point &p)
{
	Point q(p.getx(), p.gety()+1, p.getz()); 
	// compute plane equation of triangle(V0,V1,V2)
	Vector u(m_p2, m_p1);
	Vector v(m_p3, m_p1);
	Vector* n;
	n=u.cross(v);
	double d1=-n->dot((Vector)m_p1);
    // plane equation 1: n.X+d1=0 

    // put S0, S1 into plane equation 1 to compute signed distances to the plane
	double du0= n->dot((Vector)p)+d1;
	double du1= n->dot((Vector)q)+d1;

    // coplanarity robustness check 
	if(fabs(du0)<EPSILON) du0=0.0;
	if(fabs(du1)<EPSILON) du1=0.0;
	double du0du1=du0*du1;

	if(n->distance() < EPSILON) {delete n; return false;} // triangle is degenerate

	//  *idx=(du0>0)?0:1; // Which segment end is "outside" triangle?
	Vector dir(q,p); // ray direction vector
	Vector w0(p, m_p1);
	double a=n->dot(w0);
	double b=n->dot(dir);
	if (fabs(b) < EPSILON) {delete n; return false;} // ray is parallel to triangle plane

	double r = a / b;
	dir.mult(r);
	Point result=p;
	((Point)dir).addp(&result);
  // is I inside T?
    double uu, uv, vv, wu, wv, D1;
	uu = u.dot(u);
	uv = u.dot(v);
	vv = v.dot(v);

	Vector w(m_p1);
	w.mult(-1.);
	result.addp((Point*)&w);
    wu = w.dot(u);
    wv = w.dot(v);
    D1 = uu * vv -uv * uv;
	bool isInside = true;
	// get and test parametric coords
    double s, t;
    s = (uv * wv - vv * wu) / D1;
    if (s < -EPSILON || s > 1.0+EPSILON) isInside = false;  // I is outside T
    t = (uv * wu - uu * wv) / D1;
    if (t < -EPSILON || (s + t) > 1.0+EPSILON) isInside = false;  // I is outside T

    p=result; // I is in T
	delete n;
	return isInside;
}




void LdTriangle::projectY(LdObject *surf) 
{
	if(!(surf->isSurface())) return;
	surf->projectPointY(m_p1);
	surf->projectPointY(m_p2);
	surf->projectPointY(m_p3);
}

bool LdTriangle::isOutside(LdObject* line)
{
	Point* p1;
	Point* p2;
	p1=((LdLine*)line)->getP1();
	p2=((LdLine*)line)->getP2();
	if((m_p1.getYOrientation(*p1,*p2) >= 0) && 
		(m_p2.getYOrientation(*p1,*p2) >= 0) && 
		(m_p3.getYOrientation(*p1,*p2) >= 0)) return true;

	return false;
}

bool LdTriangle::SegmentIsOutside(LdObject* line)
{
	Point* p1;
	Point* p2;
	p1=((LdLine*)line)->getP1();
	p2=((LdLine*)line)->getP2();
	if(((m_p1.getYOrientation(m_p2,*p1) >= 0) &&  (m_p1.getYOrientation(m_p2,*p2) >= 0) ) || 
		((m_p2.getYOrientation(m_p3,*p1) >= 0)&&  (m_p2.getYOrientation(m_p3,*p2) >= 0) ) || 
		((m_p3.getYOrientation(m_p1,*p1) >= 0) && (m_p3.getYOrientation(m_p1,*p2) >= 0) )) return true;
	return false;
}

void LdTriangle::Cut(LdObject* line, LdObject* &o1, LdObject* &o2)
{

	int nbside = 3;

	Point p1, p2, p3;
	p1=*((LdLine*)line)->getP1();
	p2=*((LdLine*)line)->getP2();

	// calculate distances between surface vertices and cutting line
	dist[0] = m_p1.getLineDist(p1,p2);
	dist[1] = m_p2.getLineDist(p1,p2);
	dist[2] = m_p3.getLineDist(p1,p2);

	// find number of points on the cutting line, one side, or on the other
	int distnul=0;
	int distpos=0;
	
	for(int i=0; i<nbside; i++)
	{
		if(fabs(dist[i])<LdPrecision) distnul++;
		if(dist[i]>LdPrecision) distpos++;
	}
	int distneg=nbside-distnul-distpos;

	// all points on the wrong side, surface deleted
	if(distpos==0) return;
	// all points on the right side, or segment outside surface: surface not modified
	if((distpos==nbside))// || SegmentIsOutside(line) )
	{
		o1=new LdTriangle(*this);
		return;
	}
	
	// at least one vertex is on cutting line
	if(distnul>0)
	{
		// all other points on the wrong side, surface deleted
		if(distpos==0) return;
		// all other points on the right side, surface not modified
		if(distneg==0)
		{
			o1=new LdTriangle(*this);
			return;
		}

		// set m_p1 = vertex on cutting line
		for(int i=0; i<nbside; i++)
		{
			if(fabs(dist[0])<LdPrecision) break;
			RotPoints();
		}
		// if(SegmentIsOutside(line)) return;

		// Opposite side is cut
		LdLine l(m_p2,m_p3);
		l.inter((LdLine*)line, &p1);
		// return cut triangle on the right side
		if(dist[1]>LdPrecision) o1=new LdTriangle(m_p1, m_p2, p1, m_col);
		else o1=new LdTriangle(m_p1, p1, m_p3, m_col);
		return;
	}

	// only one point is on the right side, result will be a triangle
	if(distpos == 1)
	{
		// Place this point as m_p1
		for(int i=0; i<nbside; i++)
		{
			if(dist[0]>LdPrecision) break;
			RotPoints();
		}
		// Calculate intersections of adjacent sides
		LdLine l1(m_p1,m_p2);
		LdLine l2(m_p1,m_p3);
		l1.inter((LdLine*)line, &p2);
		l2.inter((LdLine*)line, &p3);
		// return triangle
		o1=new LdTriangle(m_p1, p2, p3, m_col);
		return;
	}
	// two point on the right side, result will be a quad
	// Place wrong side point as m_p1
	for(i=0; i<nbside; i++)
	{
		if(dist[0]<-LdPrecision) break;
		RotPoints();
	}
	// Calculate intersections of adjacent sides
	LdLine l1(m_p1,m_p2);
	LdLine l2(m_p1,m_p3);
	l1.inter((LdLine*)line, &p2);
	l2.inter((LdLine*)line, &p3);
	// return quad
	o1=new LdQuad(p2, m_p2, m_p3, p3, m_col);
	return;

}


bool LdTriangle::matchLine(LdObject &line)
{
	if(line.matchPoint(m_p1))
	{
		if(line.matchPoint(m_p2)) return true;
		if(line.matchPoint(m_p3)) return true;
		return false;
	}
	if(line.matchPoint(m_p2) && line.matchPoint(m_p3)) return true;
	return false;
}

Vector* LdTriangle::getNormal()
{
	Vector v1(m_p1, m_p2);
	Vector v2(m_p2, m_p3);
	Vector *v3 = v1.cross(v2);
	v3->normalize();
	return v3;
}

double LdTriangle::getAngle(LdObject &surf, LdObject &line )
{
	double result;
	Vector *v1 = getNormal();
	Vector *v2 = surf.getNormal();
	Vector *v3 = v1->cross(*v2);
	result = v3->norm();
	Vector v4(*((LdLine&)line).getP1(), *((LdLine&)line).getP2());
	if(result>.999999999) result = 90;
	else result = 180*asin(result)/PI;
	if(v4.dot(*v3) > 0) result=-result;
	if(surf.GetBFC() == -1) result=-result;
	delete v1;
	delete v2;
	delete v3;
	return result;
}

double LdTriangle::getAngle(LdObject &surf)
{
	double result;
	Vector *v1 = getNormal();
	Vector *v2 = surf.getNormal();
	Vector *v3 = v1->cross(*v2);
	result = v3->norm();
	if(result>.999999999) result = 90;
	else result = 180*asin(result)/PI;
	result = 180*asin(result)/PI;
	delete v1;
	delete v2;
	delete v3;
	return result;
}


Point* LdTriangle::getOtherPoint (LdObject &line)
{
	if(!((LdLine&)line).matchPoint(m_p1)) return &m_p1;
	if(!((LdLine&)line).matchPoint(m_p2)) return &m_p2;
	return &m_p3;
}

void LdTriangle::getOtherPoints(Point* p1, Point* p2)
{
	if((m_p1 == *p1) && (m_p2 == *p2))
	{
		p1 = new Point(m_p3);
		p2 = NULL;
	}
	else if((m_p2 == *p1) && (m_p3 == *p2))
	{
		p1 = new Point(m_p1);
		p2 = NULL;
	}
	else if((m_p3 == *p1) && (m_p1 == *p2))
	{
		p1 = new Point(m_p2);
		p2 = NULL;
	}
	else p1 = p2 = NULL;
	return;
}

void LdTriangle::InvertWinding()
{
	Point p;
	p = m_p1;
	m_p1 = m_p3;
	m_p3 = p;
}

bool LdTriangle::isInvalid(LdObject* &ldo)
{
	if((m_p1.distance(m_p2) < EPSILON) || (m_p2.distance(m_p3) < EPSILON) || (m_p1.distance(m_p3) < EPSILON))
	{
		ldo=new LdEmpty;
		return true;
	}
	return false;
}


void LdTriangle::flattenY()
{
	m_p1.flattenY();
	m_p2.flattenY();
	m_p3.flattenY();
}

void LdTriangle::addPoints (PointUnify &PU, bool fixed)
{
	PU.addPoint(m_p1, fixed);
	PU.addPoint(m_p2, fixed);
	PU.addPoint(m_p3, fixed);
}
void LdTriangle::getPoints (PointUnify &PU)
{
	PU.getPoint(m_p1);
	PU.getPoint(m_p2);
	PU.getPoint(m_p3);
}


double LdTriangle::getYOrientation()
{
	return m_p2.getYOrientation(m_p1, m_p3);
}

// Class LdQuad

LdQuad::LdQuad(const LdQuad &q)
{
	m_p1=q.m_p1;
	m_p2=q.m_p2;
	m_p3=q.m_p3;
	m_p4=q.m_p4;
	m_col=q.m_col;
}

LdQuad::LdQuad(Point &p1, Point &p2, Point &p3, Point &p4, int col)
{
	m_p1=p1;
	m_p2=p2;
	m_p3=p3;
	m_p4=p4;
	m_col=col;
}


LdQuad::LdQuad(const char *str)
{
	char*p;
	p=(char*)str;
	// eat front spaces
	while(isspace(*p)!=0) p++;
	// find next space after type
	while(isspace(*p)==0) p++;
	sscanf(p,"%d", &m_col);
	// skip color
	while(isspace(*p)!=0) p++;
	// find next space
	while(isspace(*p)==0) p++;
	// Read 1st point
	p=m_p1.pparse(p);
	p=m_p2.pparse(p);
	p=m_p3.pparse(p);
	p=m_p4.pparse(p);
}

string LdQuad::toString()
{
	char color[20];
	string result = "4 ";
	sprintf(color,"%d ",m_col);
	result += color;
	result += m_p1.toString();
	result += " ";
	result += m_p2.toString();
	result += " ";
	result += m_p3.toString();
	result += " ";
	result += m_p4.toString();
	return result;
}


LdQuad::translate(Point &p)
{
	p.addp(&m_p1);
	p.addp(&m_p2);
	p.addp(&m_p3);
	p.addp(&m_p4);
}

void LdQuad::Cut(LdObject* line, LdObject* &o1, LdObject* &o2)
{

	int nbside = 4;


	Point p1, p2, p3;
	p1=*((LdLine*)line)->getP1();
	p2=*((LdLine*)line)->getP2();

	// calculate distances between surface vertices and cutting line
	dist[0] = m_p1.getLineDist(p1,p2);
	dist[1] = m_p2.getLineDist(p1,p2);
	dist[2] = m_p3.getLineDist(p1,p2);
	dist[3] = m_p4.getLineDist(p1,p2);

	// find number of points on the cutting line, one side, or on the other
	int distnul=0;
	int distpos=0;
	
	for(int i=0; i<nbside; i++)
	{
		if(fabs(dist[i])<LdPrecision) distnul++;
		if(dist[i]>LdPrecision) distpos++;
	}
	int distneg=nbside-distnul-distpos;

	// all points on the wrong side, surface deleted
	if(distpos==0) return;
	// all points on the right side, or segment outside surface: surface not modified
	if((distpos==nbside))// || SegmentIsOutside(line) )
	{
		o1=new LdQuad(*this);
		return;
	}
	
	// at least one vertex is on cutting line
	if(distnul>0)
	{
		// all other points on the wrong side, surface deleted
		if(distpos==0) return;
		// all other points on the right side, surface not modified
		if(distneg==0)
		{
			o1=new LdQuad(*this);
			return;
		}

		// set m_p1 = vertex on cutting line
		for(int i=0; i<nbside; i++)
		{
			if(fabs(dist[0])<LdPrecision) break;
			RotPoints();
		}

		if(distnul > 1) // opposite vertices on cutting line - return 1 triangle
		{
			if(dist[1] > LdPrecision) o1 = new LdTriangle(m_p1, m_p2, m_p3, m_col);
			else o1 = new LdTriangle(m_p1, m_p3, m_p4, m_col);
			return;
		}
		// one opposite segment is cut.
		if(dist[2]*dist[1] > 0) // m_p3 m_p4 is cut
		{
			LdLine l(m_p3,m_p4);
			l.inter((LdLine*)line, &p1);
			if(dist[1] > 0) o1=new LdQuad(m_p1, m_p2, m_p3, p1, m_col);
			else o1=new LdTriangle(m_p1, p1, m_p4, m_col);
			return;
		}
		else // m_p2 m_p3 is cut
		{
			LdLine l(m_p2,m_p3);
			l.inter((LdLine*)line, &p1);
			if(dist[1] > 0) o1=new LdTriangle(m_p1, m_p2, p1, m_col);
			else o1=new LdQuad(m_p1, p1, m_p3, m_p4, m_col);
			return;
		}
	}

	if(distpos==1) // one vertice on the right side. We get a triangle
	{
		// Place this point as m_p1
		for(int i=0; i<nbside; i++)
		{
			if(dist[0]>LdPrecision) break;
			RotPoints();
		}
		// Calculate intersections of adjacent sides
		LdLine l1(m_p1,m_p2);
		LdLine l2(m_p1,m_p4);
		l1.inter((LdLine*)line, &p2);
		l2.inter((LdLine*)line, &p3);
		// return triangle
		o1=new LdTriangle(m_p1, p2, p3, m_col);
		return;
	}

	if(distneg==1) // three vertices on the right side. We get a triangle+quad
	{
		// Place this point as m_p1
		for(int i=0; i<nbside; i++)
		{
			if(dist[0]<-LdPrecision) break;
			RotPoints();
		}
		// Calculate intersections of adjacent sides
		LdLine l1(m_p1,m_p2);
		LdLine l2(m_p1,m_p4);
		l1.inter((LdLine*)line, &p2);
		l2.inter((LdLine*)line, &p3);
		// We get a pentagon. Return it as a triangle and a quad
		if(dist[1] > dist[3])
		{
			o1=new LdTriangle(p3, p2, m_p2, m_col);
			o2=new LdQuad(p3, m_p2, m_p3, m_p4, m_col);
		}
		else
		{
			o1=new LdTriangle(p3, p2, m_p4, m_col);
			o2=new LdQuad(p2, m_p2, m_p3, m_p4, m_col);
		}
		return;
	}
	// two opposite sides are cut: we get a quad
	// Place them so that we need to cut p2-p3 and p4-p1
	for(i=0; i<nbside; i++)
	{
		if(dist[0] * dist[1] > 0) break;
		RotPoints();
	}
	LdLine l1(m_p2,m_p3);
	LdLine l2(m_p1,m_p4);
	l1.inter((LdLine*)line, &p2);
	l2.inter((LdLine*)line, &p3);
	if(dist[0]>0) o1=new LdQuad(p3, m_p1, m_p2, p2, m_col);
	else o1=new LdQuad(m_p3, m_p4, p3, p2, m_col);
	return;
}

void LdQuad::swapXY()
{
	m_p1.swapXY();
	m_p2.swapXY();
	m_p3.swapXY();
	m_p4.swapXY();
}
void LdQuad::swapYZ()
{
	m_p1.swapYZ();
	m_p2.swapYZ();
	m_p3.swapYZ();
	m_p4.swapYZ();
}

void LdQuad::RotPoints(void)
{
	Point temp;
	double disttemp;
	temp = m_p1;
	m_p1=m_p2;
	m_p2=m_p3;
	m_p3=m_p4;
	m_p4=temp;
	disttemp=dist[0];
	dist[0]=dist[1];
	dist[1]=dist[2];
	dist[2]=dist[3];
	dist[3]=disttemp;
}

LdQuad::transform(Matrix &m)
{
	Vector* v1;
	v1=m.Mult((Vector)m_p1);
	m_p1=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p2);
	m_p2=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p3);
	m_p3=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p4);
	m_p4=*v1;
	delete v1;
}


LdObject* LdQuad::getFirstLine()
{
	m_edgecpt=0;
	return new LdLine(m_p1,m_p2);
}

LdObject* LdQuad::getNextLine()
{
	m_edgecpt++;
	switch (m_edgecpt)
	{
	case 1:
		return new LdLine(m_p2,m_p3);
	case 2:
		return new LdLine(m_p3,m_p4);
	case 3:
		return new LdLine(m_p4,m_p1);
	default:
		return NULL;
	}
}

bool LdQuad::projectPointY(Point &p)
{
	LdTriangle t1(m_p1, m_p2, m_p3, 0);
	LdTriangle t2(m_p1, m_p3, m_p4, 0);
	LdTriangle t3(m_p2, m_p3, m_p4, 0);
	LdTriangle t4(m_p2, m_p4, m_p1, 0);

	// Find flattest triangle decomposition
	if (t1.getAngle(t2) < t3.getAngle(t4))
	{
		if(t1.projectPointY(p)) return true;
		if(t2.projectPointY(p)) return true;
	}
	else 
	{
		if(t3.projectPointY(p)) return true;
		if(t4.projectPointY(p)) return true;
	}
	return false;
}


void LdQuad::projectY(LdObject *surf) 
{
	if(!(surf->isSurface())) return;
	surf->projectPointY(m_p1);
	surf->projectPointY(m_p2);
	surf->projectPointY(m_p3);
	surf->projectPointY(m_p4);
}


bool LdQuad::isOutside(LdObject* line)
{
	Point* p1;
	Point* p2;
	p1=((LdLine*)line)->getP1();
	p2=((LdLine*)line)->getP2();
	if((m_p1.getYOrientation(*p1,*p2) >= 0) && 
		(m_p2.getYOrientation(*p1,*p2) >= 0) && 
		(m_p3.getYOrientation(*p1,*p2) >= 0) && 
		(m_p4.getYOrientation(*p1,*p2) >= 0)) return true;
	return false;
}

bool LdQuad::matchLine(LdObject &line)
{
	if(!line.matchPoint(m_p1) && !line.matchPoint(m_p3)) return false;
	LdLine ltemp (m_p1, m_p2);
	if(ltemp.matchLine(line)) return true;
	ltemp.SetLinePoints(m_p2, m_p3);
	if(ltemp.matchLine(line)) return true;
	ltemp.SetLinePoints(m_p3, m_p4);
	if(ltemp.matchLine(line)) return true;
	ltemp.SetLinePoints(m_p4, m_p1);
	if(ltemp.matchLine(line)) return true;
	return false;
}

Vector* LdQuad::getNormal()
{
	Vector v1(m_p1, m_p2);
	Vector v2(m_p2, m_p3);
	Vector *v3 = v1.cross(v2);
	v3->normalize();
	return v3;
}

double LdQuad::getAngle(LdObject &surf, LdObject &line )
{
	double result;
	Vector *v1 = getNormal();
	Vector *v2 = surf.getNormal();
	Vector *v3 = v1->cross(*v2);
	result = v3->norm();
	Vector v4(*((LdLine&)line).getP1(), *((LdLine&)line).getP2());
	if(v4.dot(*v3) > 0) result=-result;
	if(surf.GetBFC() == -1) result=-result;
	delete v1;
	delete v2;
	delete v3;
	if(result>.999999999) return(90);
	if(result<-.999999999) return(-90);
	return 180*asin(result)/PI;
}

Point* LdQuad::getOtherPoint (LdObject &line)
{
	if(!((LdLine&)line).matchPoint(m_p1)) return &m_p1;
	if(!((LdLine&)line).matchPoint(m_p2)) return &m_p2;
	return &m_p3;
}

void LdQuad::getOtherPoints(Point* p1, Point* p2)
{
	if((m_p1 == *p1) && (m_p2 == *p2))
	{
		p1 = new Point(m_p3);
		p2 = new Point(m_p4);
	}
	else if((m_p2 == *p1) && (m_p3 == *p2))
	{
		p1 = new Point(m_p4);
		p2 = new Point(m_p1);
	}
	else if((m_p3 == *p1) && (m_p4 == *p2))
	{
		p1 = new Point(m_p1);
		p2 = new Point(m_p2);
	}
	else if((m_p4 == *p1) && (m_p1 == *p2))
	{
		p1 = new Point(m_p2);
		p2 = new Point(m_p3);
	}
	else p1 = p2 = NULL;
	return;
}

void LdQuad::InvertWinding()
{
	Point p;
	p = m_p1;
	m_p1 = m_p3;
	m_p3 = p;
}

bool LdQuad::isInvalid(LdObject* &ldo)
{
	if(m_p1.distance(m_p2) < EPSILON)
	{
		ldo=new LdTriangle(m_p2, m_p3, m_p4, m_col);
		return true;
	}
	if(m_p2.distance(m_p3) < EPSILON)
	{
		ldo=new LdTriangle(m_p1, m_p3, m_p4, m_col);
		return true;
	}
	if(m_p3.distance(m_p4) < EPSILON)
	{
		ldo=new LdTriangle(m_p1, m_p2, m_p4, m_col);
		return true;
	}
	if(m_p4.distance(m_p1) < EPSILON)
	{
		ldo=new LdTriangle(m_p1, m_p2, m_p3, m_col);
		return true;
	}
	return false;
}

void LdQuad::flattenY()
{
	m_p1.flattenY();
	m_p2.flattenY();
	m_p3.flattenY();
	m_p4.flattenY();
}

void LdQuad::addPoints (PointUnify &PU, bool fixed)
{
	PU.addPoint(m_p1, fixed);
	PU.addPoint(m_p2, fixed);
	PU.addPoint(m_p3, fixed);
	PU.addPoint(m_p4, fixed);
}
void LdQuad::getPoints (PointUnify &PU)
{
	PU.getPoint(m_p1);
	PU.getPoint(m_p2);
	PU.getPoint(m_p3);
	PU.getPoint(m_p4);
}



double LdQuad::getYOrientation()
{
	return m_p2.getYOrientation(m_p1, m_p3);
}

// Class LdCondline

LdCondline::LdCondline(const char *str)
{
	char*p;
	p=(char*)str;
	// eat front spaces
	while(isspace(*p)!=0) p++;
	// find next space after type
	while(isspace(*p)==0) p++;
	sscanf(p,"%d", &m_col);
	// skip color
	while(isspace(*p)!=0) p++;
	// find next space
	while(isspace(*p)==0) p++;
	// Read 1st point
	p=m_p1.pparse(p);
	p=m_p2.pparse(p);
	p=m_p3.pparse(p);
	p=m_p4.pparse(p);
}

LdCondline::LdCondline(Point &p1, Point &p2, Point &p3, Point &p4, const int col)
{
	m_p1=p1;
	m_p2=p2;
	m_p3=p3;
	m_p4=p4;
	m_col=col;
}


string LdCondline::toString()
{
	char color[20];
	string result = "5 ";
	sprintf(color,"%d ",m_col);
	result += color;
	result += m_p1.toString();
	result += " ";
	result += m_p2.toString();
	result += " ";
	result += m_p3.toString();
	result += " ";
	result += m_p4.toString();
	return result;
}


LdCondline::translate(Point &p)
{
	p.addp(&m_p1);
	p.addp(&m_p2);
	p.addp(&m_p3);
	p.addp(&m_p4);
}

void LdCondline::addPoints (PointUnify &PU, bool fixed)
{
	PU.addPoint(m_p1, fixed);
	PU.addPoint(m_p2, fixed);
	PU.addPoint(m_p3, fixed);
	PU.addPoint(m_p4, fixed);
}
void LdCondline::getPoints (PointUnify &PU)
{
	PU.getPoint(m_p1);
	PU.getPoint(m_p2);
	PU.getPoint(m_p3);
	PU.getPoint(m_p4);
}

void LdCondline::swapXY()
{
	m_p1.swapXY();
	m_p2.swapXY();
	m_p3.swapXY();
	m_p4.swapXY();
}
void LdCondline::swapYZ()
{
	m_p1.swapYZ();
	m_p2.swapYZ();
	m_p3.swapYZ();
	m_p4.swapYZ();
}

LdCondline::transform(Matrix &m)
{
	Vector* v1;
	v1=m.Mult((Vector)m_p1);
	m_p1=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p2);
	m_p2=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p3);
	m_p3=*v1;
	delete v1;
	v1=m.Mult((Vector)m_p4);
	m_p4=*v1;
	delete v1;
}


LdObject* LdCondline::getFirstLine()
{
	return new LdLine(m_p1,m_p2);
}

bool LdCondline::matchLine(LdObject &line)
{
	LdLine ltemp (m_p1, m_p2);
	if(ltemp.matchLine(line)) return true;
	return false;
}

bool LdCondline::isInvalid(LdObject* &ldo)
{

	if((m_p1.distance(m_p2) < EPSILON) || (m_p1.distance(m_p3) < EPSILON) 
	|| (m_p1.distance(m_p3) < EPSILON) || (m_p1.distance(m_p4) < EPSILON)
	|| (m_p2.distance(m_p3) < EPSILON) || (m_p2.distance(m_p4) < EPSILON) || (m_p3.distance(m_p4) < EPSILON))
	{
		ldo=new LdEmpty;
		return true;
	}
	return false;
}


// Class LdPath
LdPath::LdPath()
{

}

LdPath::~LdPath()
{
	for(int i=0; i<pathtable.size(); i++)
	{
		delete pathtable[i];
	}
}

LdPath::addPath(string &s)
{
	string* path;
	path=new string(s);
	pathtable.push_back(path);
}

bool LdPath::subOpen(ifstream &ldSubfile, string &subfilename)
{
	for(int i=0; i<pathtable.size(); i++)
	{
		string sfn = *pathtable[i];
		sfn += subfilename;
		// cout << sfn.c_str() << endl;
		ldSubfile.open(sfn.c_str(), ios::in);
		if(ldSubfile.good()) return true;
		ldSubfile.clear();
	}
  return false;
}

string LdPath::toString()
{
	string result;
	for(int i=0; i<pathtable.size(); i++)
	{
		result += *pathtable[i];
		if(i==pathtable.size()-1) continue;
		result += ", ";
	}
	return result;
}

// Class Fldraw

#define BUFLEN 1000

void funcdelete (LdObject* ldo) 
{
	if(ldo != NULL)	delete ldo;
}

Fldraw::Fldraw(Fldraw &fld)
{
	for (int i=0; i<fld.ldtab.size(); i++)
	{	
		switch(fld.ldtab[i]->getType())
		{
		case Comment:
			ldtab.push_back( new LdComment(*(LdComment*)(fld.ldtab[i])));
			break;
		case Subfile:
			ldtab.push_back( new LdSubfile(*(LdSubfile*)(fld.ldtab[i])));
			break;
		case Line:
			ldtab.push_back( new LdLine(*(LdLine*)(fld.ldtab[i])));
			break;
		case Triangle:
			ldtab.push_back( new LdTriangle(*(LdTriangle*)(fld.ldtab[i])));
			break;
		case Quad:
			ldtab.push_back( new LdQuad(*(LdQuad*)(fld.ldtab[i])));
			break;
		case Condline:
			ldtab.push_back( new LdCondline(*(LdCondline*)(fld.ldtab[i])));
			break;
		case Empty:
			ldtab.push_back( new LdEmpty);
			break;
		}
	}
}	


Fldraw::~Fldraw()
{
	for_each(ldtab.begin(), ldtab.end(), funcdelete);
}

bool Fldraw::ldLoad (char *fname)
{
	char buf[BUFLEN];
	int type;
	int numline=0;
	int numEdge=0;
	int numCond=0;
	int numTri=0;
	int numQuad=0;
	int numSub=0;
	int curBFC=0;
	int newBFC;
	
	// ouverture fichier	
	ifstream ldFileIn(fname, ios::in);
	if(ldFileIn == NULL) return false;

	while (!ldFileIn.eof())
	{
		ldFileIn.getline(buf, BUFLEN);
		if(ldFileIn.fail())
		{
			if(ldFileIn.eof()) break;
			ldFileIn.clear();
			continue;
		}

		if (strlen(buf)==0) continue;
		sscanf(buf,"%d ", &type);

		switch(type)
		{
		case Comment:
			char *p;
			p=strchr(buf,'0');
			if(strlen(p)<2)
				*p=0;
			else
				p+=2;
			ldtab.push_back(new LdComment(p));
			newBFC = ((LdComment*)ldtab.back())->readbfc();
			switch (newBFC)
			{
			case 2:
				curBFC = 1;
				break;
			case 3:
				curBFC = -1;
				break;
			default:
				break;
			}
			break;
		case Subfile:
			numSub++;

			ldtab.push_back(new LdSubfile(buf));
			break;
		case Line:
			numEdge++;
			if(DelEdge) break;
			ldtab.push_back(new LdLine(buf));
			break;
		case Triangle:
			numTri++;
			ldtab.push_back(new LdTriangle(buf));
			break;
		case Quad:
			numQuad++;
			ldtab.push_back(new LdQuad(buf));
			break;
		case Condline:
			numCond++;
			if(DelCond) break;
			ldtab.push_back(new LdCondline(buf));
			break;
		default:
			break;
		}
		if(ldtab.back()->getType() != Subfile)	ldtab.back()->SetBFC(curBFC);
		else ldtab.back()->SetBFC(1);
	}
	m_nLine = ldtab.size();
	cout << m_nLine << " lines in file"<< endl;
	cout << numEdge << " Edge lines";
	if(DelEdge) cout << " (deleted)";
	cout << endl << numCond << " Conditional lines";
	if(DelCond) cout << " (deleted)";
	cout << endl << numTri << " Triangles"<< endl;
	cout << numQuad << " Quads"<< endl;
	cout << numSub << " Subfiles"<< endl;

	return true;
}

void Fldraw::swapXY()
{
	for(int i=0; i<ldtab.size(); i++)
	{
		if(ldtab[i] != NULL) ldtab[i]->swapXY();
	}
}
void Fldraw::swapYZ()
{
	for(int i=0; i<ldtab.size(); i++)
	{
		if(ldtab[i] != NULL) ldtab[i]->swapYZ();
	}
}

void Fldraw::ForceYOrientation(void)
{
	for(int i=0; i<ldtab.size(); i++)
	{
		if (ldtab[i]->getYOrientation() < 0) ldtab[i]->InvertWinding();
		ldtab[i]->flattenY();
	}
}

void Fldraw::InvertWinding(void)
{
	for(int i=0; i<ldtab.size(); i++)
	{
		ldtab[i]->InvertWinding();
	}
}



Fldraw::ldExpand(LdPath *path)
{
	char buf[BUFLEN];
	int type;
	int numline=0;
	int numEdge=0;
	int numCond=0;
	int numTri=0;
	int numQuad=0;
	int numSub=0;
	int curBFC=0;
	int newBFC;
	int invertBFC;
	int invertsub;
	Point *po;
	Matrix *mat;
	int color;
	for(int i=0; i<ldtab.size(); i++)
	{
		ifstream ldSubfile;

		if(ldtab[i]->getType() != Subfile) continue;
		// cout << ldtab[i]->toString().c_str() << endl;
		po=((LdSubfile *)ldtab[i])->getOrigin();
		mat=((LdSubfile *)ldtab[i])->getMAtrix();
		color=((LdSubfile *)ldtab[i])->getColor();
		if(! path->subOpen(ldSubfile, ((LdSubfile *)ldtab[i])->getName())) continue;
		
		invertBFC = ((LdSubfile *)ldtab[i])->GetBFC();
		if(i>0)
		{
			if((ldtab[i-1]->getType() == Comment) && ((LdComment *)ldtab[i-1])->readbfc() == 1)
				invertBFC *= -1;
		}
		invertsub=invertBFC;
		if(mat->MatDeterminant() < 0) invertBFC *= -1; 
		while (!ldSubfile.eof())
		{
			ldSubfile.getline(buf, BUFLEN);
			if(ldSubfile.fail())
			{
				if(ldSubfile.eof()) break;
				ldSubfile.clear();
				continue;
			}

			if (strlen(buf)==0) continue;
			type=Empty;
			sscanf(buf,"%d ", &type);
			switch(type)
			{
			case Comment:
				char *p;
				p=strchr(buf,'0');
				if(p==NULL) break;
				if(strlen(p)<2)
					*p=0;
				else
					p+=2;
				ldtab.push_back(new LdComment(p));
				newBFC = ((LdComment*)ldtab.back())->readbfc();
				switch (newBFC)
				{
				case 2:
					curBFC = 1;
					break;
				case 3:
					curBFC = -1;
					break;
				case 1:
					break;
				default:
					if(!SubComments)
					{
						delete ldtab.back();
						ldtab.back() = new LdEmpty;
					}
					break;
				}
				break;
			case Subfile:
				numSub++;
				ldtab.push_back(new LdSubfile(buf));
				break;
			case Line:
				numEdge++;
				// if(DelEdge) break;
				ldtab.push_back(new LdLine(buf));
				break;
			case Triangle:
				numTri++;
				ldtab.push_back(new LdTriangle(buf));
				if(invertBFC == -1) ((LdTriangle*)ldtab.back())->InvertWinding();
				break;
			case Quad:
				numQuad++;
				ldtab.push_back(new LdQuad(buf));
				if(invertBFC == -1) ((LdTriangle*)ldtab.back())->InvertWinding();
				break;
			case Condline:
				numCond++;
				// if(DelCond) break;
				ldtab.push_back(new LdCondline(buf));
				break;
			default:
				break;
			}
			if(type != Empty)
			{
				ldtab.back()->SetNoOut(true);
				ldtab.back()->transform(*mat);
				ldtab.back()->translate(*po);
				if((ldtab.back()->getColor() == 16) || (ldtab.back()->getColor() == 24)) ldtab.back()->SetColor(color);
				if(ldtab.back()->getType() != Subfile)	ldtab.back()->SetBFC(curBFC);
				else ldtab.back()->SetBFC(invertsub);
			}
		}
		ldSubfile.close();
		// delete ldtab[i];
		// ldtab[i]=new LdEmpty;
		if(Inlined)
		{
			string line = "Inlined: ";
			line += ldtab[i]->toString();
			delete ldtab[i];
			if(SubComments)	ldtab[i]=new LdComment(line.c_str());
			else ldtab[i]=new LdEmpty;
			if(i>0)
			{
				if((ldtab[i-1]->getType() == Comment) && ((LdComment *)ldtab[i-1])->readbfc() == 1)
				{
					delete ldtab[i-1];
					ldtab[i-1]=new LdEmpty;
				}
			}
		}
	}
	cout << "\nInlined from subfiles:\n";
	cout << numEdge << " Edge lines";
	// if(DelEdge) cout << " (deleted)";
	cout << endl << numCond << " Conditional lines";
	// if(DelCond) cout << " (deleted)";
	cout << endl << numTri << " Triangles"<< endl;
	cout << numQuad << " Quads"<< endl;
	// m_nLine = ldtab.size();
}

bool Fldraw::ldSave (char *fname)
{
	int numline=0;
	int numEdge=0;
	int numCond=0;
	int numTri=0;
	int numQuad=0;
	int numSub=0;

	ofstream ldFileOut(fname, ios::out);
	if(ldFileOut == NULL) return false;
	for(int i=0; i<m_nLine; i++)
	{
		string s=ldtab[i]->toString();
		if(ldtab[i]->toString().length() == 0) continue;
		ldFileOut << ldtab[i]->toString().c_str() << endl;
		numline++;
		switch (ldtab[i]->getType())
		{
		case Line:
			numEdge++;
			break;
		case Triangle:
			numTri++;
			break;
		case Quad:
			numQuad++;
			break;
		case Condline:
			numCond++;
			break;
		case Subfile:
			numSub++;
			break;
		}
	}
	cout << numline << " lines written in file"<< endl;
	cout << numEdge << " Edge lines";
	cout << endl << numCond << " Conditional lines";
	cout << endl << numTri << " Triangles"<< endl;
	cout << numQuad << " Quads"<< endl;
	cout << numSub << " Subfiles"<< endl;
	return true;
}

Fldraw::findOrphan()
{
	int OrphanNumber = 0;
//	cout << "Taille avant " << ldtab.size() << endl;
	ldtab.push_back(new LdComment("// Unmatched edges"));
	for(int i=0; i<m_nLine; i++)
	{
		if(!ldtab[i]->isSurface()) continue;
		LdLine *ldl = (LdLine *)ldtab[i]->getFirstLine();
		do
		{
			bool foundmatch=false;
			for(int j=0; j<m_nLine; j++)
			{
				if(j==i) continue;
				if(ldtab[j]->matchLine(*ldl))
				{
					foundmatch=true;
					break;
				}
			}
			if(!foundmatch)
			{
				OrphanNumber++;
				ldl->SetColor(4); // Orphan lines are red
				ldtab.push_back(ldl);
			}
			else delete ldl;

		} while ((ldl = (LdLine *)ldtab[i]->getNextLine())!=NULL);
	
	}
//	cout << "Taille apres " << ldtab.size() << endl;
	cout << endl << OrphanNumber << " Unmatched edges found" << endl;
}

// test function
void Fldraw::Test(LdObject* ldo)
{
/*	for(int i=0; i<ldtab.size(); i++)
	{
		if(!ldtab[i]->Overlap(surf)) ldtab[i]->SetColor(4);
	}
*/
	LdLine *ldl = new LdLine("2 4 3 0 19 -10 -10 20");
	ldtab.push_back(ldl);
	ldtab.push_back(ldo);
	Point p1= *(ldl->getP1());
	Point p2= *(ldl->getP2());
	ldo->projectPointY(p1);
	ldo->projectPointY(p2);
	LdLine* l = new LdLine(p1,p2);
	ldtab.push_back(l);
	
}

// test function 2
void Fldraw::Test2(Fldraw* fldo)
{
	LdObject *p1=NULL, *p2=NULL;
	LdComment *bfc = new LdComment("BFC CERTIFY CCW");
	ldtab.push_back(bfc);

	LdLine *ldl = new LdLine("2 4 20 0 10 0 0 10");
	ldtab.push_back(ldl);
		
	for(int i=0; i<fldo->ldtab.size(); i++)
	{
		int a=fldo->ldtab[i]->getType();
		fldo->ldtab[i]->Cut(ldl, p1, p2);
		if(p1!=NULL) ldtab.push_back(p1);
		if(p2!=NULL) ldtab.push_back(p2);
	}
}

void Fldraw::CutAtEdges(LdObject* surf)
{
	LdObject *p1, *p2;
	LdLine *ldl = (LdLine *)surf->getFirstLine();
//	int j = 0;
	do
	{
		for(int i=0; i<ldtab.size(); i++)
		{
			p1 = p2 = NULL;
			ldtab[i]->Cut(ldl,p1,p2);
			delete ldtab[i];
			if(p1!=NULL) ldtab[i]=p1;
			else ldtab[i] = new LdEmpty;
			if(p2!=NULL) ldtab.push_back(p2);
		}
		delete ldl;
//		j++;
//		if(j==3) break;
	} while ((ldl = (LdLine *)surf->getNextLine())!=NULL);

/*
	// Cleanup any surface that might remain outside surf. 
	for(int i=0;i<ldtab.size(); i++)
	{
		if(!ldtab[i]->isSurface()) continue;
		if(!surf->Overlap(ldtab[i]))
		{
			delete ldtab[i];
			ldtab[i] = new LdEmpty;
		}
	}
*/
 }

void Fldraw::Retriangulate()
{
	Poly poly;

	int i;
	int color;
	int nbquad=0, nbtri=0;
	for(i=0;i<ldtab.size(); i++)
	{
		if(ldtab[i]->getType()==Triangle) nbtri++;
		if(ldtab[i]->getType()==Quad) nbquad++;
	}
	if(nbtri + nbquad<2) return; // no need to retriangulate
	for(i=0;i<ldtab.size(); i++)
	{
		if(!(ldtab[i]->isSurface())) continue;
		color = ldtab[i]->getColor();
		LdLine *ldl = (LdLine *)ldtab[i]->getFirstLine();
		do
		{
			Point *p = new Point(*(ldl->getP1()));
			poly.addNewVertex(p);
			delete ldl;
		} while ((ldl = (LdLine *)ldtab[i]->getNextLine())!=NULL);
	}
	for(i=0;i<ldtab.size(); i++) delete ldtab[i];
	ldtab.clear();

	poly.getBottomMost();
	poly.setAzimuth();
//	cout << "botmost" << endl;
//	cout << poly.toString().c_str();
	poly.sortAz();
//	cout << "sorted" << endl;
//	cout << poly.toString().c_str();
	poly.getConvexHull();
//	cout << "convex" << endl;
//	cout << poly.toString().c_str();

	LdObject* ldo;
	while((ldo=poly.getLdSurf(color))!=NULL)
	{
		ldtab.push_back(ldo);
	}

	return;
}

Fldraw* Fldraw::getOverlapping(LdObject* surf)
{
	Fldraw* result = new Fldraw;
	for(int i=0; i<ldtab.size(); i++)
	{
		if(ldtab[i]->Overlap(surf))
		{
			switch (ldtab[i]->getType())
			{
			case Line:
				result->ldtab.push_back(new LdLine(*(LdLine*)ldtab[i]));
				break;
			case Triangle:
				result->ldtab.push_back(new LdTriangle(*(LdTriangle*)ldtab[i]));
				break;
			case Quad:
				result->ldtab.push_back(new LdQuad(*(LdQuad*)ldtab[i]));
				break;
			case Subfile:
				result->ldtab.push_back(new LdSubfile(*(LdSubfile*)ldtab[i]));
				break;
			}
		}
	}
	return result;
}

void Fldraw::LdCopy(Fldraw* source)
{
	for(int i=0; i<source->ldtab.size(); i++)
	{
		int type=source->ldtab[i]->getType();
		ldtab.push_back(source->ldtab[i]);
	}
}

void Fldraw::CopyCond(Fldraw &source)
{
	for(int i=0; i<source.ldtab.size(); i++)
	{
		int j=source.ldtab[i]->getType();
		if(source.ldtab[i]->getType() == Condline)
		{
			ldtab.push_back( new LdCondline(*(LdCondline*)(source.ldtab[i])));
		}
	}
}

void Fldraw::Unify(void)
{
	PointUnify PU;
	LdObject *ldo;
	// Unify points of main file between them

	// points in main file: they are not fixed
	for(int i=0; i<m_nLine; i++)
	{
		ldtab[i]->addPoints(PU, false);
	}

	for(i=0; i<m_nLine; i++)
	{
		ldtab[i]->getPoints(PU);
		if(ldtab[i]->isInvalid(ldo))
		{
			delete ldtab[i];
			ldtab[i]=ldo;
			if(ldtab[i]->isInvalid(ldo))
			{
				delete ldtab[i];
				ldtab[i]=ldo;
			}
		}
	}
	
	// now snap to subparts/primitives
	PointUnify PUF;
	// add fixed points in subfiles
	for(i=m_nLine; i<ldtab.size(); i++)
	{
		if(ldtab[i]->getType() == Condline) continue;		// don't add points for condlines in subparts.
		ldtab[i]->addPoints(PUF, true);
	}

	for(i=0; i<m_nLine; i++)
	{
		ldtab[i]->getPoints(PUF);
		if(ldtab[i]->isInvalid(ldo))
		{
			delete ldtab[i];
			ldtab[i]=ldo;
			if(ldtab[i]->isInvalid(ldo))
			{
				delete ldtab[i];
				ldtab[i]=ldo;
			}
		}
	}
}		


Fldraw* Fldraw::CutAll(Fldraw &form, Fldraw &form3D)
{
	Fldraw* result = new Fldraw;
	for(int i=0; i<form.ldtab.size(); i++)
	{
		Fldraw facet; 
		LdObject* ldo;
		ldo = form.ldtab[i];
		if(ldo->isSurface()) 
		{
			Fldraw* overlapping = new Fldraw;
			overlapping = getOverlapping(ldo);
			for(int j=0; j<overlapping->ldtab.size(); j++)
			{
				if(overlapping->ldtab[j]->getType()==Subfile) 
				{
					facet.ldtab.push_back(overlapping->ldtab[j]);
				}
				else
				{
					Fldraw patElement;
					patElement.ldtab.push_back(overlapping->ldtab[j]);
					patElement.CutAtEdges(ldo);
					if(!NoReTriangulate) patElement.Retriangulate();
					facet.LdCopy(&patElement);
					patElement.ldtab.clear();
				}
			}
			overlapping->ldtab.clear();
			delete overlapping;
			bool noMerge = false;
			if(Merge)
			{
				int color=-1;
				j=0;
				while(j<facet.ldtab.size()) 
				{
					if(facet.ldtab[j]->getType()==Subfile)
					{
						noMerge=true;
						break;
					}
					if(facet.ldtab[j]->isSurface())
					{
						color=facet.ldtab[j]->getColor();
						break;
					}
					j++;
				} 

				if(!noMerge && (color >= 0))
				{
					for(; j<facet.ldtab.size(); j++)
					{
						if(facet.ldtab[j]->getType()==Subfile)
						{
							noMerge=true;
							break;
						}
						if(facet.ldtab[j]->isSurface())
						{
							if (facet.ldtab[j]->getColor() != color)
							{
								noMerge=true;
								break;
							}
						}
					}	
				}
				if(Merge && !noMerge)
				{
					int type;
					facet.ldtab.clear();
					ldo=form.ldtab[i];
					type=ldo->getType();
					if(type==Triangle)
					{
						LdTriangle* ldt;
						ldt=(LdTriangle*)ldo;
						ldo = (LdObject*)new LdTriangle(*ldt);
					}
					if(type==Quad)
					{
						LdQuad* ldq;
						ldq=(LdQuad*)ldo;
						ldo = (LdObject*)new LdQuad(*ldq);
					}
					ldo->SetColor(color);
					facet.ldtab.push_back(ldo);
				}
			}

			if(!NoProject)
			{
				ldo = form3D.ldtab[i];
				for(j=0; j<facet.ldtab.size(); j++)
				{
					facet.ldtab[j]->projectY(ldo);
				}
			}
			result->LdCopy(&facet);
		}
		facet.ldtab.clear();
	}
	return result;
}


Fldraw::findAdjacent()
{
	int addLines=0;
	int addCond=0;
	int addInBetween=0;
	int Curcolor;
//	cout << "Taille avant " << ldtab.size() << endl;
	ldtab.push_back(new LdComment("// Added lines/Condlines"));
	for(int i=0; i<m_nLine; i++)
	{
		if(!ldtab[i]->isSurface()) continue;
		LdLine *ldl = (LdLine *)ldtab[i]->getFirstLine();
		do
		{
			bool foundmatch=true;
			for(int j=i+1; j<m_nLine; j++)
			{
				if(ldtab[j]->matchLine(*ldl))
				{
					for(int k=0; k<m_nLine; k++)
					{
						if(!ldtab[k]->isLine()) continue;
						if(ldtab[k]->matchLine(*ldl))
						{
							foundmatch=false;
							break;
						}
					}
					if(foundmatch)
					{
						double angle=ldtab[i]->getAngle(*ldtab[j], (LdObject&)*ldl);
						// cout << endl << angle << " degrees" << endl;
						if(fabs(angle) < LdFlat) 
						{
							delete ldl; 
							break;
						}
						if (fabs(angle) > LdMinEdge)
						{
							// Added lines are pink or edge color
							if(DebugColors) Curcolor=13; else Curcolor = 24;
							addLines++;
							ldl->SetColor(Curcolor); 
							ldtab.push_back(ldl);
							break;
						}
						if (fabs(angle) < LdMaxCond)
						{
							// Added condlines are light blue or edge color
							if(angle * CxCv > 0) break; 
							if(DebugColors) Curcolor=9; else Curcolor = 24;
							addCond++;
							ldtab.push_back(new LdCondline(*(ldl->getP1()), *(ldl->getP2()), 
								*(ldtab[i]->getOtherPoint(*ldl)), *(ldtab[j]->getOtherPoint(*ldl)), Curcolor));
							delete ldl;
							break;
						}

						if(angle * CxCv > 0) break; 
						addInBetween++;
						ldl->SetColor(2); // Added in-between lines are bright green
						ldtab.push_back(ldl);
						ldtab.push_back(new LdCondline(*(ldl->getP1()), *(ldl->getP2()), 
							*(ldtab[i]->getOtherPoint(*ldl)), *(ldtab[j]->getOtherPoint(*ldl)), 2)); // Added condlines are green
						break;
					}
				}
			}
			if(!foundmatch) delete ldl;
		} while ((ldl = (LdLine *)ldtab[i]->getNextLine())!=NULL);
	}
	cout << endl << addLines << " New edges found" << endl;
	cout << addCond << " New conditional lines found" << endl;
	cout << addInBetween << " New dubious lines/condlines found" << endl;
}



